@@ -18,7 +18,8 @@ pub trait Montgomery: Sized {
     /// Montgomery form on multi-precision integer representations.
     type Inv;
 
-    /// The type of integer with doubled width
+    /// The type of integer with double width. It is only used in `reduce()`,
+    /// so it's okay that it's not actually doubled with
     type Double;
 
     /// Calculate -(m^-1) mod R
@@ -62,7 +63,9 @@ macro_rules! impl_uprim_montgomery {
     () => {
         #[inline]
         fn transform(target: Self, m: &Self) -> Self {
-            if target == 0 { return 0; }
+            if target == 0 {
+                return 0;
+            }
             (((target as Self::Double) << Self::BITS) % (*m as Self::Double)) as _
         }
 
@@ -156,6 +159,8 @@ impl Montgomery for u16 {
 
     fn neginv(m: &Self) -> Self {
         let i = BINVERT_TABLE[((m >> 1) & 0x7F) as usize] as u16;
+        // Newton-Rhapson iteration
+        // See: https://arxiv.org/abs/1303.0328
         i.wrapping_mul(*m).wrapping_sub(2).wrapping_mul(i)
     }
 
@@ -168,7 +173,6 @@ impl Montgomery for u32 {
 
     fn neginv(m: &Self) -> Self {
         let i = BINVERT_TABLE[((m >> 1) & 0x7F) as usize] as u32;
-        // iteratively i = 2*i - i*i*m;
         let i = 2u32.wrapping_sub(i.wrapping_mul(*m)).wrapping_mul(i);
         i.wrapping_mul(*m).wrapping_sub(2).wrapping_mul(i)
     }
@@ -190,6 +194,9 @@ impl Montgomery for u64 {
     impl_uprim_montgomery!();
 }
 
+// XXX: implement Montgomery for u128 (double type is also u128), which requires efficient implementation of dual word mul_mod.
+// REF: https://github.com/coreutils/coreutils/blob/master/src/factor.c (mulredc2)
+
 /// An integer represented in Montgomery form, it implements [ModularInteger] interface
 /// and it's generally more efficient than the vanilla integer in modular operations.
 #[derive(Debug, Clone, Copy)]
@@ -201,12 +208,12 @@ pub struct MontgomeryInt<T: Integer + Montgomery> {
     m: T,
 
     /// The negated modular inverse of the modulus mod R
-    mi: T::Inv
+    mi: T::Inv,
 }
 
 /// A big integer represented in Montgomery form, it implements [ModularInteger] interface
 /// and it's generally more efficient than the vanilla integer in modular operations.
-/// 
+///
 /// The modulus is stored in heap to prevent frequent copying
 #[derive(Debug, Clone)]
 pub struct MontgomeryBigint<T: Integer + Montgomery> {
@@ -292,7 +299,11 @@ impl<T: Integer + Montgomery> Add for MontgomeryInt<T> {
     fn add(self, rhs: Self) -> Self::Output {
         self.check_modulus_eq(&rhs);
         let a = Montgomery::add(&self.a, &rhs.a, &self.m);
-        MontgomeryInt { a, m: self.m, mi: self.mi }
+        MontgomeryInt {
+            a,
+            m: self.m,
+            mi: self.mi,
+        }
     }
 }
 
@@ -315,7 +326,11 @@ impl<T: Integer + Montgomery> Sub for MontgomeryInt<T> {
     fn sub(self, rhs: Self) -> Self::Output {
         self.check_modulus_eq(&rhs);
         let a = Montgomery::sub(&self.a, &rhs.a, &self.m);
-        MontgomeryInt { a, m: self.m, mi: self.mi }
+        MontgomeryInt {
+            a,
+            m: self.m,
+            mi: self.mi,
+        }
     }
 }
 
@@ -337,7 +352,11 @@ impl<T: Integer + Montgomery> Neg for MontgomeryInt<T> {
     #[inline]
     fn neg(self) -> Self::Output {
         let a = Montgomery::neg(&self.a, &self.m);
-        MontgomeryInt { a, m: self.m, mi: self.mi }
+        MontgomeryInt {
+            a,
+            m: self.m,
+            mi: self.mi,
+        }
     }
 }
 
@@ -359,7 +378,11 @@ impl<T: Integer + Montgomery> Mul for MontgomeryInt<T> {
     fn mul(self, rhs: Self) -> Self::Output {
         self.check_modulus_eq(&rhs);
         let a = Montgomery::mul(&self.a, &rhs.a, &self.m, &self.mi);
-        MontgomeryInt { a, m: self.m, mi: self.mi }
+        MontgomeryInt {
+            a,
+            m: self.m,
+            mi: self.mi,
+        }
     }
 }
 
@@ -381,7 +404,11 @@ impl<T: Integer + Montgomery> Pow<T> for MontgomeryInt<T> {
     #[inline]
     fn pow(self, rhs: T) -> Self::Output {
         let a = Montgomery::pow(&self.a, &rhs, &self.m, &self.mi);
-        MontgomeryInt { a, m: self.m, mi: self.mi }
+        MontgomeryInt {
+            a,
+            m: self.m,
+            mi: self.mi,
+        }
     }
 }
 
@@ -426,7 +453,8 @@ where
 
 impl<T: Integer + Montgomery + Clone> ModularInteger for MontgomeryInt<T>
 where
-    T::Double: From<T>, T::Inv: Clone
+    T::Double: From<T>,
+    T::Inv: Clone,
 {
     type Base = T;
 
@@ -443,6 +471,10 @@ where
     #[inline]
     fn new(&self, n: T) -> Self {
         let a = Montgomery::transform(n, &self.m);
-        MontgomeryInt { a, m: self.m.clone(), mi: self.mi.clone() }
+        MontgomeryInt {
+            a,
+            m: self.m.clone(),
+            mi: self.mi.clone(),
+        }
     }
 }
