@@ -1,7 +1,6 @@
 use crate::{ModularCoreOps, ModularOps};
 use num_integer::Integer;
 
-
 macro_rules! impl_powm_uprim {
     ($T:ty) => {
         fn powm(self, exp: $T, m: &$T) -> $T {
@@ -23,11 +22,21 @@ macro_rules! impl_powm_uprim {
                 }
             }
         }
-    }
+    };
 }
 
 macro_rules! impl_jacobi_uprim {
     ($T:ty) => {
+        #[inline]
+        fn legendre(self, n: &$T) -> i8 {
+            match self.powm((n - 1) >> 1, &n) {
+                0 => 0,
+                1 => 1,
+                x if x == n - 1 => -1,
+                _ => panic!("n is not prime!"),
+            }
+        }
+
         fn jacobi(self, n: &$T) -> i8 {
             if n % 2 == 0 || n < &0 {
                 panic!("The Jacobi symbol is only defined for non-negative odd integers!")
@@ -220,7 +229,7 @@ impl ModularCoreOps<u128, &u128> for u128 {
         }
         result
     }
-    
+
     #[inline]
     fn negm(self, m: &u128) -> u128 {
         let x = self % m;
@@ -268,6 +277,10 @@ macro_rules! impl_mod_arithm_by_deref {
                 ModularOps::<$T, &$T>::invm(*self, m)
             }
             #[inline]
+            fn legendre(self, n: &$T) -> i8 {
+                ModularOps::<$T, &$T>::legendre(*self, n)
+            }
+            #[inline]
             fn jacobi(self, n: &$T) -> i8 {
                 ModularOps::<$T, &$T>::jacobi(*self, n)
             }
@@ -306,6 +319,10 @@ macro_rules! impl_mod_arithm_by_deref {
                 ModularOps::<$T, &$T>::invm(self, m)
             }
             #[inline]
+            fn legendre(self, n: &$T) -> i8 {
+                ModularOps::<$T, &$T>::legendre(self, n)
+            }
+            #[inline]
             fn jacobi(self, n: &$T) -> i8 {
                 ModularOps::<$T, &$T>::jacobi(self, n)
             }
@@ -344,6 +361,10 @@ macro_rules! impl_mod_arithm_by_deref {
                 ModularOps::<$T, &$T>::invm(*self, m)
             }
             #[inline]
+            fn legendre(self, n: &$T) -> i8 {
+                ModularOps::<$T, &$T>::legendre(*self, n)
+            }
+            #[inline]
             fn jacobi(self, n: &$T) -> i8 {
                 ModularOps::<$T, &$T>::jacobi(*self, n)
             }
